package com.dzeio.openhealth.ui.weight

import android.content.SharedPreferences
import android.graphics.Paint
import android.os.Bundle
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import androidx.core.view.MenuProvider
import androidx.navigation.fragment.findNavController
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.charts.Entry
import com.dzeio.charts.axis.Line
import com.dzeio.charts.series.LineSerie
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.WeightAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.databinding.FragmentListWeightBinding
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Date
import java.util.Locale

@AndroidEntryPoint
class ListWeightFragment :
    BaseFragment<ListWeightViewModel, FragmentListWeightBinding>(ListWeightViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) ->
    FragmentListWeightBinding = FragmentListWeightBinding::inflate

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Menu
        requireActivity().addMenuProvider(object : MenuProvider {
            override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
                menu.findItem(R.id.action_add).isVisible = true
            }

            override fun onMenuItemSelected(item: MenuItem): Boolean {
                return when (item.itemId) {
                    R.id.action_add -> {
                        findNavController().navigate(
                            ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                                WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                            )
                        )
                        true
                    }

                    else -> true
                }
            }
        })

        if (viewModel.goalWeight.value != null) {
            binding.goalButton.setText(R.string.edit_goal)
        }

        binding.goalButton.setOnClickListener {
            findNavController().navigate(
                ListWeightFragmentDirections.actionNavListWeightToNavWeightDialog(
                    WeightDialog.DialogTypes.EDIT_GOAL.ordinal
                )
            )
        }

        val adapter = WeightAdapter().apply {
            onItemClick = {
                findNavController().navigate(
                    ListWeightFragmentDirections.actionNavListWeightToNavEditWeight(
                        it.id
                    )
                )
            }
        }

        binding.list.apply {
            val manager = LinearLayoutManager(requireContext())
            layoutManager = manager
            this.adapter = adapter
        }

        viewModel.massUnit.observe(viewLifecycleOwner) {
            adapter.unit = it
        }

        viewModel.weights.observe(viewLifecycleOwner) {
            if (it != null) {
                val itt = it.toMutableList()
                itt.sortWith { o1, o2 -> if (o1.timestamp > o2.timestamp) -1 else 1 }
                adapter.set(itt)
                updateGraph(it)
            }
        }

        val chart = binding.chart

        val serie = LineSerie(chart).apply {
            linePaint.color = MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorPrimary
            )
            textPaint.color = MaterialColors.getColor(
                requireView(),
                com.google.android.material.R.attr.colorOnPrimary
            )
        }

        chart.apply {
            series = arrayListOf(serie)

            yAxis.apply {
                textLabel.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                linePaint.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )

                onValueFormat = { value -> "${value.toInt()}" }
            }

            xAxis.apply {
                // 7 day history
                textPaint.color = MaterialColors.getColor(
                    requireView(),
                    com.google.android.material.R.attr.colorOnPrimaryContainer
                )
                textPaint.textSize = 32f
                onValueFormat = onValueFormat@{
                    val formatter = DateFormat.getDateTimeInstance(
                        DateFormat.SHORT,
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }
            }
        }

        // Debug button
//        if (binding.debugRandomValues != null) {
//            binding.debugRandomValues.setOnClickListener {
//                viewModel.generateRandomValues()
//            }
//            binding.debugRandomValues.setOnLongClickListener {
//                viewModel.delete(viewModel.weights.value!!)
//                return@setOnLongClickListener true
//            }
//        }
    }

    private fun updateGraph(list: List<Weight>) {
        val chart = binding.chart
        val serie = chart.series[0] as LineSerie

        val entries: ArrayList<Entry> = arrayListOf()

        list.forEach {
            entries.add(
                Entry(
                    it.timestamp.toDouble(),
                    it.weight
                )
            )
        }
        serie.entries = entries

        if (viewModel.goalWeight.value != null) {
            chart.yAxis.addLine(
                viewModel.goalWeight.value!!,
                Line(true, Paint(chart.yAxis.linePaint).apply { strokeWidth = 4f })
            )
        }

        if (list.isEmpty()) {
            chart.xAxis.x = 0.0
        } else {
            chart.xAxis.x = list[0].timestamp.toDouble()
        }

        chart.refresh()
    }
}
